/***************************************************************************
 *   Copyright (C) 2005 by Mike Diehl   *
 *   madpenguin8@yahoo.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qdatetime.h>
#include <qfile.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpainter.h>
#include <qpopupmenu.h>
#include <qslider.h>
#include <qspinbox.h>
#include <qtooltip.h>

#include <kapplication.h>
#include <kaboutdata.h>
#include <kconfig.h>
#include <kdebug.h>
#include <kdialogbase.h>
#include <kfiledialog.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <kio/netaccess.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kpixmapio.h>
#include <kpopupmenu.h>
#include <kpushbutton.h>
#include <kstandarddirs.h>
#include <kstatusbar.h>

#include "solseek.h"
#include "imageviewer.h"

const char *description = I18N_NOOP("A panel applet for observing current solar conditions.");

SolSeeK::SolSeeK(const QString& configFile, Type type, int actions, QWidget *parent, const char *name)
    : KPanelApplet(configFile, type, actions, parent, name)
{
    sconfig = config();
    m_choices << "SOHO EIT Fe IX, X 171A" << "SOHO EIT Fe XII 195A"
        << "SOHO EIT Fe XV 284A" << "SOHO EIT He II 304A" << "SOHO MDI Continuum"
        << "SOHO MDI Magnetogram" << "SOHO LASCO C2" << "SOHO LASCO C3"
        << "Ca II K Mauna Loa Observatory" << "H-Alpha Disc Mauna Loa Observatory"
        << "H-Alpha Limb Mauna Loa Observatory" << "GOES-12 Soft X-Ray";
        
    URLs << "http://sohowww.nascom.nasa.gov/data/realtime/eit_171/%1/latest.gif"
        << "http://sohowww.nascom.nasa.gov/data/realtime/eit_195/%1/latest.gif"
        << "http://sohowww.nascom.nasa.gov/data/realtime/eit_284/%1/latest.gif"
        << "http://sohowww.nascom.nasa.gov/data/realtime/eit_304/%1/latest.gif"
        << "http://sohowww.nascom.nasa.gov/data/realtime/mdi_igr/%1/latest.gif"
        << "http://sohowww.nascom.nasa.gov/data/realtime/mdi_mag/%1/latest.gif"
        << "http://sohowww.nascom.nasa.gov/data/realtime/c2/%1/latest.gif"
        << "http://sohowww.nascom.nasa.gov/data/realtime/c3/%1/latest.gif"
        << "http://umbra.nascom.nasa.gov/images/latest_K_line.gif"
        << "http://mlso.hao.ucar.edu/latest.disk.gif"
        << "http://mlso.hao.ucar.edu/latest.limb.gif"
        << "http://150.144.30.154/~gurman/images/sxi/images/latest_sxi_pthnb.gif";
        
    QStringList viewerQueue;
    QStringList viewerNames;

    loadConfig();
    m_wxindex = 1;
    m_isAnimating = false;
    m_isAvailable = false;
    appletUpdateTimer = new QTimer( this );
    appletUpdateTimer->start( 1000 * 60 * m_timeout );
    connect( appletUpdateTimer, SIGNAL( timeout() ), SLOT( update() ) );
    animationUpdateTimer = new QTimer( this );
    animationUpdateTimer->start( 1000 * 60 * 60 * m_animateTimeout );
    connect( animationUpdateTimer, SIGNAL( timeout() ), SLOT( updateFrame() ) );
    animationSpeedTimer = new QTimer( this );
    connect( animationSpeedTimer, SIGNAL( timeout() ), SLOT( incrementFrame() ) );
    viewerAnimationSpeedTimer = new QTimer( this );
    connect( viewerAnimationSpeedTimer, SIGNAL( timeout() ), SLOT( incrementViewerFrame() ) );
    loadImage();
}


SolSeeK::~SolSeeK()
{
    thumb->remove();    // Clean temp files.
    minithumb->remove();
}


void SolSeeK::about()
{
    KDialogBase *dialog = new KDialogBase(
                                i18n("About SolSeeK"),
                                KDialogBase::Yes,
                                KDialogBase::Yes, KDialogBase::Yes,
                                0, "about", true, true,
                                KStdGuiItem::ok() );

    QPixmap ret = DesktopIcon("solseek");
    QString text = i18n( description ) + QString::fromLatin1("\n\n") +
            i18n("Written by Mike Diehl <madpenguin8@yahoo.com>\n");
    dialog->setIcon(ret);

    KMessageBox::createKMessageBox( dialog, ret, text, QStringList(), QString::null, 0, KMessageBox::Notify );
}


void SolSeeK::help()
{
    kapp->invokeHelp( QString::null, "solseek" );
}


void SolSeeK::loadConfig()
{
    sconfig->setGroup( "General" );
    m_selection = sconfig->readNumEntry( "Selection" , 0 ); //Type of image
    m_size = sconfig->readNumEntry( "Size", 256 );              // ImageViewer size.
    m_tipsize = sconfig->readNumEntry( "Tip Size", 128 );   // Tooltip preview size
    m_timeout = sconfig->readNumEntry( "Timeout", 30 ); // Update interval
    m_animateTimeout = sconfig->readNumEntry( "Animate_Timeout", 60 ); //Animation frame save interval
    m_animationSpeed = sconfig->readNumEntry( "Animate_Speed", 10 ); //Animation speed
    m_frameCount = sconfig->readNumEntry( "Frames", 40 ); //Number of frames to use during animation.
}


void SolSeeK::saveConfig()
{
    sconfig->setGroup( "General" );
    sconfig->writeEntry( "Selection", m_selection );
    sconfig->writeEntry( "Size", m_size );
    sconfig->writeEntry( "Tip Size", m_tipsize );
    sconfig->writeEntry( "Timeout", m_timeout );
    sconfig->writeEntry( "Animate_Timeout", m_animateTimeout );
    sconfig->writeEntry( "Animate_Speed", m_animationSpeed );
    sconfig->writeEntry( "Frames", m_frameCount );
    sconfig->sync();
}


void SolSeeK::preferences()
{
    KDialogBase dialog( this, 0, false );
    dialog.setCaption( kapp->makeStdCaption( i18n( "Configure SolSeeK" ) ) );
    dialog.showButtonApply( false );
    QFrame *box = dialog.makeMainWidget();

    QGridLayout *grid = new QGridLayout( box, 7, 2 );
    grid->setAutoAdd( true );

    QLabel *choice = new QLabel( box );
    choice->setText( i18n( "Image to monitor:" ) );
    QComboBox *mon = new QComboBox( false, box );
    mon->setEditable( false );
    mon->insertStringList( m_choices, 0 );
    mon->setCurrentItem( m_selection );
    QToolTip::add( mon, i18n( "Select which image to use in the applet." ) );

    QLabel *size = new QLabel( box );
    size->setText( i18n( "Size of viewable images:" ) );
    QSpinBox *sizes = new QSpinBox( 256, 1024, 8, box );
    sizes->setSuffix( i18n( "px") );
    sizes->setValue( m_size );
    QToolTip::add( sizes, i18n( "Select the size of images in the image viewer." ) );

    QLabel *tsize = new QLabel( box );
    tsize->setText( i18n( "Size of tooltip images:" ) );
    QSpinBox *tsizes = new QSpinBox( 64, 256, 2, box );
    tsizes->setSuffix( i18n( "px" ) );
    tsizes->setValue( m_tipsize );
    QToolTip::add( tsizes, i18n( "Select the size of images in the tooltip." ) );
        
    QLabel *utime = new QLabel( box );
    utime->setText( i18n( "Update Interval:" ) );
    QSpinBox *utimes = new QSpinBox( 5, 60, 5, box );
    utimes->setSuffix( i18n( "min") );
    utimes->setValue( m_timeout );
    QToolTip::add( utimes, i18n( "Select the applet update time." ) );

    QLabel *animation = new QLabel( box );
    animation->setText( i18n( "Frames for animation:" ) );
    QSpinBox *frames = new QSpinBox( 5, 100, 1, box );
    frames->setValue( m_frameCount );
    QToolTip::add( frames, i18n( "Select the number of frames to use during animation." ) );

    QLabel *atime = new QLabel( box );
    atime->setText( i18n( "Frame Update Interval:" ) );
    QSpinBox *atimes = new QSpinBox( 1, 24, 1, box );
    atimes->setSuffix( i18n( "hrs") );
    atimes->setValue( m_animateTimeout );
    QToolTip::add( atimes, i18n( "Select the frame update time." ) );

    QLabel *aspeed = new QLabel( box );
    aspeed->setText( i18n( "Animation Speed:" ) );
    QSlider *aslider = new QSlider( 2, 60, 2, m_animationSpeed, Qt::Horizontal, box );
    QToolTip::add( aslider, i18n( "Set animation speed." ) );

    if( dialog.exec() != QDialog::Rejected ){
        m_selection = mon->currentItem();
        m_size = sizes->value();
        m_tipsize = tsizes->value();
        m_timeout = utimes->value();
        m_animateTimeout = atimes->value();
        m_frameCount = frames->value();
        m_animationSpeed = aslider->value();
        appletUpdateTimer->changeInterval( 1000 * 60 * m_timeout );
        animationUpdateTimer->changeInterval( 1000 * 60 * 60 * m_animateTimeout );
        saveConfig();
        loadImage();
    }

}


int SolSeeK::widthForHeight( int height ) const
{
    return height;
}


int SolSeeK::heightForWidth( int width ) const
{
    return width;
}


void SolSeeK::update()
{
    loadImage();
}


void SolSeeK::paintEvent( QPaintEvent* e )
{
    if ( !buffer.isNull() && !m_isAnimating )
        bitBlt( this, e->rect().topLeft(), &buffer, e->rect() );
}


void SolSeeK::loadImage()
{
    KURL url;
    
    if ( m_selection < 8 ){
        url = QString( URLs[ m_selection ] ).arg( "256" );
        credit = "SOHO (ESA and NASA)";
    }
    
    else if ( m_selection < 11 ){
        url = QString( URLs[ m_selection ] );
        credit = "Mauna Loa Solar Observatory";
    }
        
    else {
        url = QString( URLs[ m_selection ] );
        credit = "NOAA / SEC";
    }
    
    thumb = tempfile.file();
    tempfile.unlink();

    KURL saveURL ( thumb->name() );
    if ( !saveURL.isValid() )
        kdDebug()<<"tempfile-URL is malformed\n";

    downloadJob = KIO::file_copy ( url, saveURL, -1, true, false, false );
    connect ( downloadJob, SIGNAL ( result (KIO::Job *) ), SLOT ( downloadReady (KIO::Job *) ) );
}


void SolSeeK::downloadReady(KIO::Job *job)
{
// set downloadJob to 0, but don't delete it - the job will automatically deleted !!!
    downloadJob = 0;

    if ( job->error() )
    {
        m_isAvailable = false;
        showImage();
        animationUpdateTimer->stop(); //Don't update the animation frames with offline cache.
        kdDebug()<<"Animation updates disabled.\n";
       return;		// exit this function
    }

    thumb->close(); // to get the newest informations of the file and not any informations from opening of the file

    if ( thumb->exists() )
    {
        m_isAvailable = true;
        showImage();
        if ( !animationUpdateTimer->isActive() ) //Restart the timer if it was stopped last time.
            animationUpdateTimer->start( 1000 * 60 * 60 * m_animateTimeout );
        return;
    }
}


void SolSeeK::showImage()
{

    QImage image;
            
    KPixmapIO kpix;

    KIconLoader *ldr = KGlobal::iconLoader();        
    QPixmap icon;
    icon = ldr->loadIcon( "connect_no", KIcon::Small );  
   
    if ( m_isAvailable ){
        image.load( thumb->name() );
        image.save( imageCacheURL(), "PNG", 100 );
        buffer = kpix.convertToPixmap( image.smoothScale( width(), height(), QImage::ScaleMin ) );
    }
    else{
        image.load( imageCacheURL() );
        buffer = kpix.convertToPixmap( image.smoothScale( width(), height(), QImage::ScaleMin ) );
    }
        
        QPainter p;
        QRect r( contentsRect() );

        if( !m_isAvailable ){
            p.begin( &buffer );
            p.drawPixmap( 1, 1, icon );
            p.end();
        }    

        p.begin( this );
        p.drawPixmap( r.x(), r.y(), buffer );
        p.end();

        minithumb = minifile.file();
        minifile.unlink();
        QPixmap mini;
        mini = kpix.convertToPixmap( image.smoothScale( m_tipsize, m_tipsize, QImage::ScaleMin ) );
        mini.save( minithumb->name(), "PNG" );

        QString tip;
        tip += QString( "<center><b>%1</b></center>" ).arg( QString( m_choices[ m_selection ] ) );
        tip += QString( "<center><img src='%1'></center>" ).arg( minithumb->name() );
        tip += QString( "<center><i>%1</i></center>" ).arg( credit );

        QToolTip::add( this, tip );

}

void SolSeeK::updateFrame() //Slot
{
    saveFrame( m_frameCount );
}

void SolSeeK::saveFrame( int maxFrames )
{
    QString saveURL = saveFramePath() + "/" + QString( m_choices[ m_selection ] ) + "_" + QString::number( width() ) + ".png";
    saveURL.replace( " ", "_" );
    
    KPixmapIO fpix;

    QPixmap *result = new QPixmap(); //Buffer for drawing frames.

    QImage framesImg;
    if ( framesImg.load( saveURL ) ){

        QPixmap frames;
        frames = fpix.convertToPixmap( framesImg );
        
        if ( frames.height() < ( height() * maxFrames ) )
            result->resize( width(), frames.height() + buffer.height() ); //Only make it as big as we can.
        else
            result->resize( width(),  ( height() * maxFrames ) );

        QPainter p;
        p.begin( result );
        p.drawPixmap( 0, 0, buffer ); //Draw the newest frame first.
        p.drawPixmap( 0, height(), frames ); //Then append as many of the rest as we can.
        p.end();
        result->save( saveURL, "PNG" );
    }
    else
        buffer.save( saveURL, "PNG" ); //Save the first frame.
}

void SolSeeK::startAnimation()
{
    m_isAnimating = true;

    QString saveURL = saveFramePath() + "/" + QString( m_choices[ m_selection ] ) + "_" + QString::number( width() ) + ".png";
    saveURL.replace( " ", "_" );

    if ( savedFrames.load( saveURL ) ){  //Load saved frames and start timer.
        m_currentFrame = savedFrames.height() / savedFrames.width();  //Set the number of frames.
        animationSpeedTimer->start( 1000 / m_animationSpeed );
    }
    else
        stopAnimation();
}

void SolSeeK::stopAnimation()
{
    m_isAnimating = false;
    animationSpeedTimer->stop();
    bitBlt( this, 0, 0, &buffer, 0, 0 );
}

void SolSeeK::incrementFrame()
{
    if ( m_currentFrame > 0 ){
        m_currentFrame--;
        QPainter p;
        QRect r( contentsRect() );

        p.begin( this );
        p.drawPixmap( r.x(), r.y(), savedFrames, 0, ( m_currentFrame * height() ), width(), height() );
        p.end();
    }
    else
        stopAnimation();
}

void SolSeeK::downloadViewerFrames()
{
    viewerQueue = URLs;
    
    viewerNames = m_choices;
    
    viewerDownload();
}

void SolSeeK::viewerDownload()
{
    if( !viewerQueue.isEmpty() ){
        QString viewerURL;
    
        if( QString( viewerQueue[ 0 ] ).contains( "sohowww" ) ) //Only the soho images need a size argument.
            viewerURL = QString( viewerQueue[ 0 ] ).arg( "256" );
        else
            viewerURL = QString( viewerQueue[ 0 ] );
        
        viewerFrame = tempfile.file();
        tempfile.unlink();

        KURL saveURL ( viewerFrame->name() );
        if ( !saveURL.isValid() )
            kdDebug()<<"tempfile-URL is malformed\n";

        viewerFrameJob = KIO::file_copy ( viewerURL, saveURL, -1, true, false, false );
        connect ( viewerFrameJob, SIGNAL ( result (KIO::Job *) ), SLOT ( viewerDownloadReady (KIO::Job *) ) );
    }
    else
        kdDebug()<<"Viewer downloads complete.\n";
}

void SolSeeK::viewerDownloadReady(KIO::Job *job)
{
// set downloadJob to 0, but don't delete it - the job will automatically deleted !!!
    viewerFrameJob = 0;

    if ( job->error() )
    {
        viewerQueue.pop_front();
        viewerNames.pop_front();
        viewerDownload();
        return;
    }

    viewerFrame->close();
    
    if ( viewerFrame->exists() )
    {
        saveViewerFrame( QString( viewerNames[ 0 ] ), 20  );
        return;
    }
}

void SolSeeK::saveViewerFrame( QString type, int maxFrames )
{

    QString saveURL = saveViewerFramePath() + "/" + type + ".png";
    saveURL.replace( " ", "_" );
    
    KPixmapIO fpix;

    QPixmap *result = new QPixmap(); //Buffer for drawing frames.
    
    QImage viewerImage;
    QPixmap viewerPixmap;
    
    viewerImage.load( viewerFrame->name() );
    viewerPixmap = fpix.convertToPixmap( viewerImage.smoothScale( 256, 256 ) );
    
    QImage framesImg;
    if ( framesImg.load( saveURL ) ){

        QPixmap frames;
        frames = fpix.convertToPixmap( framesImg );
        
        if ( frames.height() < ( 256 * maxFrames ) )
            result->resize( 256, frames.height() + 256 ); //Only make it as big as we can.
        else
            result->resize( 256,  ( 256 * maxFrames ) );

        QPainter p;
        p.begin( result );
        p.drawPixmap( 0, 0, viewerPixmap ); //Draw the newest frame first.
        p.drawPixmap( 0, 256, frames ); //Then append as many of the rest as we can.
        p.end();
        result->save( saveURL, "PNG" );
    }
    else
        viewerPixmap.save( saveURL, "PNG" ); //Save the first frame.
        
    viewerQueue.pop_front();
    viewerNames.pop_front();

    viewerDownload();
}

void SolSeeK::animationViewer()
{
    KDialogBase dialog( 0, 0, false, i18n( "SolSeeK Animation Viewer" ), 0 );
    QFrame *box = dialog.makeMainWidget();

    currentViewerType = QString( m_choices[ m_selection ] );
    m_viewerSpeed = m_animationSpeed;
    
    
    KIconLoader *ldr = KGlobal::iconLoader();        
    QPixmap icon;
    icon = ldr->loadIcon( "solseek", KIcon::Small, KIcon::SizeEnormous );  

    viewerBuffer.resize( 256, 256 );
    
    viewerLabel = new QLabel( box );
    viewerLabel->setMaximumSize( 256, 256 );
    viewerLabel->setMinimumSize( 256, 256 );
    viewerLabel->setScaledContents( true );
    viewerLabel->setPixmap( icon );
   
    QVBox *vbox = new QVBox( box );
    vbox->move( 0, viewerLabel->height() + 1 );
    vbox->setMinimumWidth( viewerLabel->width() );
    QComboBox *mon = new QComboBox( false, vbox );
    mon->setEditable( false );
    mon->insertStringList( m_choices, 0 );
    mon->setCurrentItem( m_selection );
    QToolTip::add( mon, i18n( "Select the type of animation to view." ) );
    QSlider *slider = new QSlider( 2, 60, 2, m_animationSpeed, Qt::Horizontal, vbox );
    QToolTip::add( slider, i18n( "Animation speed." ) );
    vbox->setMinimumHeight( mon->height() + slider->height() );

    QHBox *buttonBox = new QHBox( box );
    buttonBox->move( 0, viewerLabel->height() + vbox->height() + 1 );
    buttonBox->setMinimumWidth( viewerLabel->width() );
    buttonBox->setSpacing( 1 );
    QPushButton *play = new QPushButton( SmallIconSet( "player_play" ), i18n( "Play" ), buttonBox );
    QPushButton *pause = new QPushButton( SmallIconSet( "player_pause" ), i18n( "Pause" ), buttonBox );
    QPushButton *stop = new QPushButton( SmallIconSet( "player_stop" ), i18n( "Stop"), buttonBox );
    
    box->setMinimumWidth( viewerLabel->width() );
    box->setMinimumHeight( vbox->height() + viewerLabel->height() + buttonBox->height() );

    connect( mon, SIGNAL( activated( const QString & ) ), this, SLOT( setFileForViewer( const QString & ) ) );
    connect( slider, SIGNAL( valueChanged(int) ), this, SLOT( setViewerSpeed(int) ) );
    connect( play, SIGNAL( clicked() ), this, SLOT( startViewerAnimation() ) );
    connect( pause, SIGNAL( clicked() ), this, SLOT( pauseViewerAnimation() ) );
    connect( stop, SIGNAL( clicked() ), this, SLOT( stopViewerAnimation() ) );

    
    dialog.disableResize();
    dialog.show();
    dialog.exec();
    stopViewerAnimation();

}

void SolSeeK::setViewerSpeed( int speed )
{
    if( viewerAnimationSpeedTimer->isActive() ) 
        viewerAnimationSpeedTimer->changeInterval( 1000 / speed );
    else
        m_viewerSpeed = speed;    
}

void SolSeeK::setFileForViewer( const QString &type )
{
    currentViewerType = type;
}

void SolSeeK::startViewerAnimation()
{
    QString saveURL = saveViewerFramePath() + "/" + currentViewerType + ".png";
    saveURL.replace( " ", "_" );

    if ( savedViewerFrames.load( saveURL ) ){ 
        m_currentViewerFrame = savedViewerFrames.height() / savedViewerFrames.width();  //Set the number of frames.
        m_viewerFrameCount = m_currentViewerFrame; //For looping.
        viewerAnimationSpeedTimer->start( 1000 / m_viewerSpeed );
    }
    else{
        viewerLabel->setText( i18n( "Sorry, animation file was not found. Please try using the Update Animation Viewer option in the right click menu." ) );
        stopViewerAnimation();
    }
}

void SolSeeK::pauseViewerAnimation()
{
    if( viewerAnimationSpeedTimer->isActive() ){
        viewerAnimationSpeedTimer->stop();
        viewerLabel->setPixmap( viewerBuffer );
    }
    else
        viewerAnimationSpeedTimer->start( 1000 / m_viewerSpeed );
}

void SolSeeK::stopViewerAnimation()
{
    viewerAnimationSpeedTimer->stop();
    viewerLabel->setPixmap( viewerBuffer );
}

void SolSeeK::incrementViewerFrame()
{
    if ( m_currentViewerFrame > 0 ){
        m_currentViewerFrame--;
        QPainter p;

        p.begin( viewerLabel );
        p.drawPixmap( 0, 0, savedViewerFrames, 0, ( m_currentViewerFrame * 256 ), 256, 256 );
        p.end();
        
        p.begin( &viewerBuffer );
        p.drawPixmap( 0, 0, savedViewerFrames, 0, ( m_currentViewerFrame * 256 ), 256, 256 );
        p.end();
    }
    else{
        m_currentViewerFrame = m_viewerFrameCount; //Looping.
 /*       stopViewerAnimation();
        KIconLoader *ldr = KGlobal::iconLoader();        
        QPixmap icon;
        icon = ldr->loadIcon( "solseek", KIcon::Small, KIcon::SizeLarge );  
        viewerLabel->setPixmap( icon );
        */
    }
}

void SolSeeK::mousePressEvent( QMouseEvent *e )
{
    enum { FEIX, FEXII, FEXV, HEII, CONTINUUM, MAGNETO, CTWO, CTHREE, SOHO, HALPHAD, HALPHAL, CAK, MAUNA,
                WX, ANIMATE, SAVEF, GOES, BIG, VIEWER };
    KPopupMenu *popup = new KPopupMenu(this);
    popup->insertTitle( SmallIcon("solseek"), i18n("View"));

    KPopupMenu *sohoMenu = new KPopupMenu( this );
    sohoMenu->insertTitle( SmallIcon( "solseek" ), "SOHO" );
    sohoMenu->insertItem( SmallIcon( "solseek" ), i18n( "EIT 171A" ), FEIX );
    sohoMenu->insertItem( SmallIcon( "solseek" ), i18n( "EIT 195A" ), FEXII );
    sohoMenu->insertItem( SmallIcon( "solseek" ), i18n( "EIT 284A" ), FEXV );
    sohoMenu->insertItem( SmallIcon( "solseek" ), i18n( "EIT 304A" ), HEII );
    sohoMenu->insertItem( SmallIcon( "solseek" ), i18n( "MDI Continuum" ), CONTINUUM );
    sohoMenu->insertItem( SmallIcon( "solseek" ), i18n( "MDI Magnetogram" ), MAGNETO );
    sohoMenu->insertItem( SmallIcon( "solseek" ), i18n( "LASCO C2" ), CTWO );
    sohoMenu->insertItem( SmallIcon( "solseek" ), i18n( "LASCO C3" ), CTHREE );
    popup->insertItem( SmallIcon( "solseek" ), "SOHO", sohoMenu, SOHO );

    KPopupMenu *maunaMenu = new KPopupMenu( this );
    maunaMenu->insertItem( SmallIcon("solseek" ), i18n("H-Alpha Disc"), HALPHAD);
    maunaMenu->insertItem( SmallIcon("solseek" ), i18n("H-Alpha Limb"), HALPHAL);
    maunaMenu->insertItem( SmallIcon("solseek" ), i18n("Calcium K"), CAK);
    popup->insertItem( SmallIcon("solseek" ), "Mauna Loa", maunaMenu, MAUNA );
    popup->insertItem( SmallIcon("solseek" ), i18n("GOES-12 Soft X-Ray" ), GOES );
    popup->insertItem( SmallIcon("solseek" ), i18n("Space Weather" ), WX);
    popup->insertItem( SmallIcon("solseek" ), i18n( "Animate Applet" ), ANIMATE );
    popup->insertItem( SmallIcon("solseek" ), i18n( "Save Current Frame" ), SAVEF );
    popup->setItemEnabled( SAVEF, m_isAvailable );
    popup->insertItem( SmallIcon("solseek" ), i18n( "Animation Viewer" ), VIEWER );
    popup->insertItem( SmallIcon("solseek" ), i18n( "Update Animation Viewer" ), BIG );

    if ( !popup )
      return;

    if ( e->button() == RightButton ) {
         popup->popup( mapToGlobal( e->pos() ) );
        QString url;
        QString msg;

        QString minsize;
        if ( m_size > 640 ) // Download smallest possible image then downscale, but allow some upscale.
            minsize = "1024";
        else if ( m_size > 376 )
            minsize = "512";
        else
            minsize = "256";

        switch (popup->exec())
        {
           case FEIX:{
                url = QString( "http://sohowww.nascom.nasa.gov/data/realtime/eit_171/%1/latest.gif" ).arg( minsize );
                msg = "SOHO (ESA and NASA) EIT Fe IX, X 171A";
            }
                break;
            case FEXII:{
                url = QString( "http://sohowww.nascom.nasa.gov/data/realtime/eit_195/%1/latest.gif" ).arg( minsize );
                msg = "SOHO (ESA and NASA) EIT Fe XII 195A";
            }
                break;
            case FEXV:{
                url = QString( "http://sohowww.nascom.nasa.gov/data/realtime/eit_284/%1/latest.gif" ).arg( minsize );
                msg = "SOHO (ESA and NASA) EIT Fe XV 284A";
            }
                break;
            case HEII:{
                url =QString( "http://sohowww.nascom.nasa.gov/data/realtime/eit_304/%1/latest.gif" ).arg( minsize );
                msg = "SOHO (ESA and NASA) EIT He II 304A";
            }
                break;
            case CONTINUUM:{
                url = QString( "http://sohowww.nascom.nasa.gov/data/realtime/mdi_igr/%1/latest.gif" ).arg( minsize );
                msg = "SOHO (ESA and NASA) MDI Continuum";
            }
                break;
            case MAGNETO:{
                url = QString( "http://sohowww.nascom.nasa.gov/data/realtime/mdi_mag/%1/latest.gif" ).arg( minsize );
                msg = "SOHO (ESA and NASA) MDI Magnetogram";
            }
                break;
            case CTWO:{
                url = QString( "http://sohowww.nascom.nasa.gov/data/realtime/c2/%1/latest.gif" ).arg( minsize );
                msg = "SOHO (ESA and NASA) LASCO C2";
            }
                break;
            case CTHREE:{
                url = QString( "http://sohowww.nascom.nasa.gov/data/realtime/c3/%1/latest.gif" ).arg( minsize );
                msg = "SOHO (ESA and NASA) LASCO C3";
            }
                break;
            case CAK:{
                url = "http://umbra.nascom.nasa.gov/images/latest_K_line.gif";
                msg = "Ca II K Mauna Loa Solar Observatory";
            }
                break;
            case HALPHAD:{
                url = "http://mlso.hao.ucar.edu/latest.disk.gif";
                msg = "H-Alpha Disc Mauna Loa Solar Observatory";
            }
                break;
            case HALPHAL:{
                url = "http://mlso.hao.ucar.edu/latest.limb.gif";
                msg = "H-Alpha Limb Mauna Loa Solar Observatory";
            }
                break;
            case GOES:{
                url = "http://150.144.30.154/~gurman/images/sxi/images/latest_sxi_pthnb.gif";
                msg = "GOES-12 Soft X-Ray";
            }
                break;
            case WX:
                downloadWx();
                break;
            case ANIMATE:
                startAnimation();
                break;
            case VIEWER:
                animationViewer();
                break;
            case SAVEF:
                saveFrame( m_frameCount );
                break;
            case BIG:
                downloadViewerFrames();
                break;
        }
        if ( !url.isEmpty() ){
            KURL kurl = KURL::fromPathOrURL( url );
            new ImageViewer( &kurl, msg, m_size, 0 );
        }
    }

    if ( e->button() == MidButton )
      preferences(); 

    if ( e->button() == LeftButton )
      startAnimation();

}

QString SolSeeK::saveWxPath()
{
    QString path = KGlobal::dirs()->saveLocation( "data", "solseek/wxpixmaps", true );
    path += "/";
    return path;
}

QString SolSeeK::saveFramePath()
{
    return KGlobal::dirs()->saveLocation( "data", "solseek/animation_pixmaps", true );
}

QString SolSeeK::saveViewerFramePath()
{
    return KGlobal::dirs()->saveLocation( "data", "solseek/big_animation_pixmaps", true );
}

QString SolSeeK::imageCacheURL()
{
    QString path = KGlobal::dirs()->saveLocation( "data", "solseek/latest_pixmaps", true );
    path += "/";
    path += QString( m_choices[ m_selection ] );
    path += ".png";
    return path.replace( " ", "_" );
}

void SolSeeK::downloadWx()
{
    int wxnumber = 1;

    QStringList args;
    args << "sw_densi.jpg" << "sw_speed.jpg" << "sw_pressure.jpg" << "temperature.jpg"
        << "imf_mag.jpg" << "imf_polar.jpg" << "imf_azimuth.jpg" << "convection.jpg"
        << "alfven_speed.jpg" << "sound_speed.jpg" << "thermal_energy.jpg" << "log_beta.jpg"
        << "alfven_mach.jpg" << "mach_number.jpg";

    for( QStringList::Iterator it = args.begin(); it != args.end(); ++it ){
        QString save = saveWxPath() + "/" + QString::number( wxnumber ) + ".jpg";
        KURL url = KURL::fromPathOrURL( QString( "http://space.rice.edu/ISTP/dials/%1" ).arg( *it ) );
        KURL saveURL = KURL::fromPathOrURL( save );
        KIO::Job *job = KIO::file_copy ( url, saveURL, -1, true, false, false );
        connect( job , SIGNAL (result (KIO::Job *)), SLOT (downloadWxReady (KIO::Job *)));
        wxnumber++;
    }
}

void SolSeeK::downloadWxReady(KIO::Job *job)
{
    m_wxindex++;

    if ( job->error() )
    {
        job->showErrorDialog();
        return;     // exit this function
    }

    if ( m_wxindex > 14 ){
        kdDebug()<<"All downloads finished\n";
        showWx();
        m_wxindex = 1;
    }
}

void SolSeeK::showWx()
{
    KDialogBase dialog( 0, 0, false, i18n("SolSeeK"), 0  );
    QFrame *box = dialog.makeMainWidget();
    box->setMaximumSize( 141 * 5, 161 * 3 );

    QGridLayout *grid = new QGridLayout( box, 3, 5 );
    grid->setAutoAdd( true );

    QImage image;
    KPixmapIO kpix;

    QLabel *wx1 = new QLabel( box );
    image.load( saveWxPath() + "1.jpg" );
    QPixmap w1 = kpix.convertToPixmap( image );
    wx1->setPixmap( w1 );

    QLabel *wx2 = new QLabel( box );
    image.load( saveWxPath() + "2.jpg" );
    QPixmap w2 = kpix.convertToPixmap( image );
    wx2->setPixmap( w2 );

    QLabel *wx3 = new QLabel( box );
    image.load( saveWxPath() + "3.jpg" );
    QPixmap w3 = kpix.convertToPixmap( image );
    wx3->setPixmap( w3 );

    QLabel *wx4 = new QLabel( box );
    image.load( saveWxPath() + "4.jpg" );
    QPixmap w4 = kpix.convertToPixmap( image );
    wx4->setPixmap( w4 );

    QLabel *wx5 = new QLabel( box );
    image.load( saveWxPath() + "5.jpg" );
    QPixmap w5 = kpix.convertToPixmap( image );
    wx5->setPixmap( w5 );

    QLabel *wx6 = new QLabel( box );
    image.load( saveWxPath() + "6.jpg" );
    QPixmap w6 = kpix.convertToPixmap( image );
    wx6->setPixmap( w6 );

    QLabel *wx7 = new QLabel( box );
    image.load( saveWxPath() + "7.jpg" );
    QPixmap w7 = kpix.convertToPixmap( image );
    wx7->setPixmap( w7 );

    QLabel *wx8 = new QLabel( box );
    image.load( saveWxPath() + "8.jpg" );
    QPixmap w8 = kpix.convertToPixmap( image );
    wx8->setPixmap( w8 );

    QLabel *wx9 = new QLabel( box );
    image.load( saveWxPath() + "9.jpg" );
    QPixmap w9 = kpix.convertToPixmap( image );
    wx9->setPixmap( w9 );

    QLabel *wx10 = new QLabel( box );
    image.load( saveWxPath() + "10.jpg" );
    QPixmap w10 = kpix.convertToPixmap( image );
    wx10->setPixmap( w10 );

    QLabel *wx11 = new QLabel( box );
    image.load( saveWxPath() + "11.jpg" );
    QPixmap w11 = kpix.convertToPixmap( image );
    wx11->setPixmap( w11 );

    QLabel *wx12 = new QLabel( box );
    image.load( saveWxPath() + "12.jpg" );
    QPixmap w12 = kpix.convertToPixmap( image );
    wx12->setPixmap( w12 );

    QLabel *wx13 = new QLabel( box );
    image.load( saveWxPath() + "13.jpg" );
    QPixmap w13 = kpix.convertToPixmap( image );
    wx13->setPixmap( w13 );

    QLabel *wx14 = new QLabel( box );
    image.load( saveWxPath() + "14.jpg" );
    QPixmap w14 = kpix.convertToPixmap( image );
    wx14->setPixmap( w14 );

    dialog.disableResize();
    dialog.show();
    dialog.exec();
}

extern "C"
{
    KPanelApplet* init( QWidget *parent, const QString& configFile)
    {
        KGlobal::locale()->insertCatalogue("solseek");
        return new SolSeeK(configFile, KPanelApplet::Normal,
                             KPanelApplet::About | KPanelApplet::Help | KPanelApplet::Preferences,
                             parent, "solseek");
    }
}
