/***************************************************************************
 *   Copyright (C) 2005 by Mike Diehl   *
 *   madpenguin8@yahoo.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SOLSEEK_H
#define SOLSEEK_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qimage.h>
#include <qlabel.h>
#include <qpixmap.h>
#include <qstring.h>
#include <qtimer.h>
#include <qvbox.h>
#include <kconfig.h>
#include <kio/job.h>
#include <kpanelapplet.h>
#include <kprogress.h>
#include <ktempfile.h>
#include <kurl.h>


class SolSeeK : public KPanelApplet
{
    Q_OBJECT

public:
    SolSeeK(const QString& configFile, Type t = Normal, int actions = 0,
        QWidget *parent = 0, const char *name = 0);
    /** destructor */
    ~SolSeeK();

    virtual int widthForHeight(int height) const;
    virtual int heightForWidth(int width) const;
    virtual void about();
    virtual void help();
    virtual void preferences();
    virtual void paintEvent( QPaintEvent* e );

public slots:
    void update();

protected:
    void mousePressEvent( QMouseEvent *e );

private:
    bool m_isAnimating;
    bool m_isAvailable;
    void loadConfig();
    void loadImage();
    void downloadWx();
    void showImage();
    void showWx();
    void saveConfig();
    void saveFrame( int maxFrames );
    void downloadViewerFrames();
    void viewerDownload();
    void saveViewerFrame( QString type, int maxFrames );
    void animationViewer();
    int m_selection;
    int m_size;
    int m_tipsize;
    int m_timeout;
    int m_animateTimeout;
    int m_animationSpeed;
    int m_viewerSpeed;
    int m_wxindex;
    int m_currentFrame;
    int m_currentViewerFrame;
    int m_frameCount;
    int m_viewerFrameCount;
    KConfig *sconfig;
    KTempFile tempfile;
    KTempFile minifile;
    QFile *thumb;
    QFile *viewerFrame;
    QFile *minithumb;
    QLabel *viewerLabel;
    QPixmap buffer;
    QPixmap viewerBuffer;
    QPixmap savedFrames;
    QPixmap savedViewerFrames;
    QString credit;
    QString currentViewerType;
    QString saveWxPath();
    QString saveFramePath();
    QString saveViewerFramePath();
    QString imageCacheURL();
    QStringList viewerQueue;
    QStringList viewerNames;
    QStringList m_choices;
    QStringList URLs;
    QTimer *appletUpdateTimer;
    QTimer *animationUpdateTimer;
    QTimer *animationSpeedTimer;
    QTimer *viewerAnimationSpeedTimer;
    KIO::Job *downloadJob;
    KIO::Job *viewerFrameJob;

    private slots:
    /**Make sure download has finished, then make sure file exists, then display the image */
    void downloadReady( KIO::Job * );
    void viewerDownloadReady( KIO::Job * );
    void downloadWxReady( KIO::Job * );
    void updateFrame();
    void startAnimation();
    void stopAnimation();
    void incrementFrame();
    void startViewerAnimation();
    void pauseViewerAnimation();
    void stopViewerAnimation();
    void incrementViewerFrame();
    void setFileForViewer( const QString &name );
    void setViewerSpeed( int speed );
};

#endif
